import base64
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.backends import default_backend
from cryptography.fernet import Fernet, InvalidToken
import sys
from colorama import Fore, Style, init
init(autoreset=True)
import os

def fernet_encrypt(data, password):
    # generate encryption KEY base on the password provided!
    hash_obj = hashes.Hash(hashes.SHA256(), backend=default_backend())
    hash_obj.update(password.encode())
    key = base64.urlsafe_b64encode(hash_obj.finalize())

    # encrypt the data
    fernet = Fernet(key)
    encrypted_message = fernet.encrypt(data.encode('utf-8'))

    # return encrypted data
    return encrypted_message.decode()


def fernet_decrypt(data, password):
    # generate encryption KEY base on the password provided!
    hash_obj = hashes.Hash(hashes.SHA256(), backend=default_backend())
    hash_obj.update(password.encode())
    key = base64.urlsafe_b64encode(hash_obj.finalize())

    # encrypt the data
    fernet = Fernet(key)
    try:
        encrypted_message = fernet.decrypt(data.encode('utf-8'))
    except InvalidToken:
        print(Fore.RED + "Provided password is wrong! Try again...")
        sys.exit()

    # return encrypted data
    return encrypted_message.decode()


def save_data(data, path):

    with open(path, 'w') as file:
            file.write(data)
    return path


def read_data(path):
    try:
        with open(path, 'r', encoding='utf-8') as file:
            data = file.read()
        return data
    except FileExistsError:
        print(Fore.RED + "Error: Path you've givent does not exist!")
        sys.exit()
    except FileNotFoundError:
        print(Fore.RED + "Error: Path you've givent is not available!!")
        sys.exit()
    except UnicodeDecodeError:
        print(Fore.RED + Style.BRIGHT + "ENC still can not encrypt this kind of file!\nThis feature will be added soon...")
        sys.exit()

def path_manager(path):
    # check if the given path is a file and exists
    if not os.path.isfile(path) or not os.path.exists(path):
        print(Fore.RED + f"Your given path: {path}, Does not exists or is not a file!")
        sys.exit()
    
    file_path = os.path.abspath(path)

    return [os.path.dirname(file_path) + "/", os.path.basename(file_path)]
    


from encrypter_decrypter import enc_tool as enc
import argparse
import json
# Just styles
from colorama import Fore, Style, init
init(autoreset=True)

# read data from config file
with open("encrypter_decrypter/config/config.json", "r") as file:
    config = json.load(file)


def main():
    parser = argparse.ArgumentParser(prog= config['name'], 
                                    description= config['description'])
    parser.add_argument('data', nargs='?', help="The message you want to encrypt.")
    parser.add_argument('-p', '--password', help="Password to encrypt or decrypt your data.", required=True)
    parser.add_argument('-d', '--decrypt', help="Add this for decryption", action="store_true")
    parser.add_argument('-s', '--save', help="Path to file that encrypted / decrypted data will be saved in.")
    parser.add_argument('-f', '--file', help="Path to file that you want encrypt / decrypt data in it.")
    args = parser.parse_args()

    if not args.decrypt:
        if args.file:
            data = enc.read_data(args.file)
            enc_data = enc.fernet_encrypt(data, args.password)

            path = enc.path_manager(args.file)
            file_path = args.save or f'{path[0]}encrypted_{path[1]}'
            enc.save_data(enc_data, file_path)
            print(Fore.BLUE + "Your data was encrypted succesfully.", 
              Style.BRIGHT + Fore.WHITE + f"Encrypted data saved in '{file_path}'",
              Style.DIM + f"Your password key: {args.password}",
              sep="\n")
        else:
            if not args.data:
                parser.error("Provide a data to encrypt!")

            enc_data = enc.fernet_encrypt(args.data, args.password)
            
            if args.save:
                enc.save_data(enc_data, args.save)
                print(Fore.BLUE + "Your data was encrypted succesfully.", 
                Style.BRIGHT + Fore.WHITE + f"Encrypted data saved in '{args.save}'",
                Style.DIM + f"Your password key: {args.password}",
                sep="\n")
            else:
                print(Fore.BLUE + "Your data was encrypted succesfully.", 
                    Style.BRIGHT + Fore.WHITE + enc_data,
                    Style.DIM + f"Your password key: {args.password}",
                    sep="\n")


    elif args.decrypt:
        if args.file:
            data = enc.read_data(args.file)
            dec_data = enc.fernet_decrypt(data, args.password)

            path = enc.path_manager(args.file)
            file_path = args.save or f'{path[0]}decrypted_{path[1]}'
            enc.save_data(dec_data, file_path)
            print(Fore.GREEN + "Your data was decrypted succesfully.", 
              Style.BRIGHT + Fore.WHITE + f"Decrypted data saved in '{file_path}'",
              sep="\n")
        
        else:
            if not args.data:
                parser.error("Provide a data to decrypt!")
            dec_data = enc.fernet_decrypt(args.data, args.password)

            if args.save:
                enc.save_data(dec_data, args.save)
                print(Fore.GREEN + "Your data was decrypted succesfully.", 
                    Style.BRIGHT + Fore.WHITE + f"Decrypted data saved in '{args.save}'",
                    sep="\n")
            else:
                print(Fore.GREEN + "Your data was decrypted succesfully.", 
                    Style.BRIGHT + Fore.WHITE + dec_data,
                    sep="\n")
                
    else:
        parser.error("Provide a message or file to encrypt or decrypt!")

if __name__ == '__main__':
    main()
    


from setuptools import setup, find_packages
import os
import json

directory = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

# read data from config file
with open(os.path.join(directory, "encrypter_decrypter/config/config.json"), "r") as file:
    config = json.load(file)

setup(
    name=config['name'],
    version= config['version'],
    packages=find_packages(),
    description= config['description'],
    long_description= long_description,
    long_description_content_type="text/markdown",
    author= config['author'],
    author_email= config['author_email'],

    entry_points={
        'console_scripts': [
            'enc = encrypter_decrypter.main:main',
        ],
    },
    install_requires=[
        'colorama', 'cryptography',
    ],
)
